#include "ProgressBar.h"
#include <Configuration.h>
#include <BasicAbstractions/Assets.h>
#include <Themes/Themes.h>
#include <States/GameState/Context/GameContext.h>
#include <string>

ProgressBar::ProgressBar(const size_t food_number) {
    const float bg_width = static_cast<float>(config::GAME_VIDEO_MODE.width)/4.f;
    const float bg_height = static_cast<float>(config::GAME_VIDEO_MODE.height)/30.f;

    m_bar_background.setSize({bg_width, bg_height});
    m_bar_background.setPosition(0, 0);
    m_bar_background.setFillColor(Themes::Instance().get_progress_bar_background());

    m_defeats_count_background.setSize({bg_width, bg_height});
    m_defeats_count_background.setPosition(
        static_cast<float>(config::GAME_VIDEO_MODE.width) - bg_width,
        0
    );
    m_defeats_count_background.setFillColor(Themes::Instance().get_progress_bar_background());

    const float text_size = bg_height * 0.6f;

    m_text_before_bar.setFont(Assets::GetFont());
    m_text_before_bar.setCharacterSize(static_cast<unsigned int>(text_size));
    m_text_before_bar.setString(config::TEXT_ON_PROGRESS_BAR);

    const sf::FloatRect text_before_bounds = m_text_before_bar.getLocalBounds();
    m_text_before_bar.setOrigin(
        text_before_bounds.left + text_before_bounds.width/2.0f,
        text_before_bounds.top + text_before_bounds.height/2.0f
    );

    m_text_before_bar.setPosition(
        bg_width/27.f + text_before_bounds.width/2.0f,
        bg_height/2
    );
    m_text_before_bar.setFillColor(Themes::Instance().get_text_on_progress_bar());

    m_progress_max_width = static_cast<float>(bg_width) - 1.2f * m_text_before_bar.getGlobalBounds().width;
    const float progress_height = bg_height * 0.7f;

    m_bar.setSize({m_progress_max_width, progress_height});
    m_bar.setPosition(
        bg_width/27.f + m_text_before_bar.getLocalBounds().width,
        bg_height/2 - progress_height/2
    );
    m_bar.setFillColor(Themes::Instance().get_progress_bar_fill());

    m_progress_bar.setSize({0, progress_height});
    m_progress_bar.setPosition(
        bg_width/27.f + m_text_before_bar.getLocalBounds().width,
        bg_height/2 - progress_height/2
    );
    m_progress_bar.setFillColor(Themes::Instance().get_progress_bar());

    m_text_on_bar.setFont(Assets::GetFont());
    m_text_on_bar.setCharacterSize(static_cast<unsigned int>(text_size));
    m_text_on_bar.setString("0/" + std::to_string(food_number));

    const sf::FloatRect text_bounds = m_text_on_bar.getLocalBounds();
    m_text_on_bar.setOrigin(
        text_bounds.left + text_bounds.width/2.0f,
        text_bounds.top + text_bounds.height/2.0f
    );

    m_text_on_bar.setPosition(
        m_progress_bar.getPosition().x + bg_width * 0.6f,
        m_progress_bar.getPosition().y + m_progress_bar.getSize().y/2.0f
    );
    m_text_on_bar.setFillColor(Themes::Instance().get_text_on_progress_bar());

    m_defeats_count_text.setFont(Assets::GetFont());
    m_defeats_count_text.setCharacterSize(static_cast<unsigned int>(text_size));
    m_defeats_count_text.setString(std::string(config::TEXT_ON_PROGRESS_BAR_DEFEATS) + "0");

    const sf::FloatRect defeats_text_bounds = m_defeats_count_text.getLocalBounds();
    m_defeats_count_text.setOrigin(
        defeats_text_bounds.left + defeats_text_bounds.width/2.0f,
        defeats_text_bounds.top + defeats_text_bounds.height/2.0f
    );
    m_defeats_count_text.setPosition(
        m_defeats_count_background.getPosition().x + bg_width/2.0f,
        m_defeats_count_background.getPosition().y + bg_height/2.0f
    );
    m_defeats_count_text.setFillColor(Themes::Instance().get_text_on_progress_bar());
}

void ProgressBar::update_food_progress_bar(const GameContext& context) {
    float progress_width;
    if (context.food_number == 0)
        progress_width = m_progress_max_width;
    else progress_width = static_cast<float>(context.eaten_food_number) / static_cast<float>(context.food_number) *
                                 m_progress_max_width;
    m_progress_bar.setSize({progress_width, m_progress_bar.getSize().y});

    m_text_on_bar.setString(std::to_string(context.eaten_food_number) + "/" + std::to_string(context.food_number));

    const sf::FloatRect text_bounds = m_text_on_bar.getLocalBounds();
    m_text_on_bar.setOrigin(
        text_bounds.left + text_bounds.width/2.0f,
        text_bounds.top + text_bounds.height/2.0f
    );
}

void ProgressBar::update_defeat_progress_bar(const GameContext& game_context) {
    m_defeats_count_text.setString(std::string(config::TEXT_ON_PROGRESS_BAR_DEFEATS) + std::to_string(game_context.defeats_count));
    const sf::FloatRect defeats_text_bounds = m_defeats_count_text.getLocalBounds();
    m_defeats_count_text.setOrigin(
        defeats_text_bounds.left + defeats_text_bounds.width / 2.0f,
        defeats_text_bounds.top + defeats_text_bounds.height / 2.0f
    );
    m_defeats_count_text.setPosition(
        m_defeats_count_background.getPosition().x + m_defeats_count_background.getSize().x / 2.0f,
        m_defeats_count_background.getPosition().y + m_defeats_count_background.getSize().y / 2.0f
    );
}

void ProgressBar::draw_into(sf::RenderWindow& window) const {
    window.draw(m_bar_background);
    window.draw(m_bar);
    window.draw(m_progress_bar);
    window.draw(m_text_before_bar);
    window.draw(m_text_on_bar);

    window.draw(m_defeats_count_background);
    window.draw(m_defeats_count_text);
}